<?php
namespace TYPO3\Gss\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Arek van Schaijk <info@ucreation.nl>, Ucreation
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
 
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 *
 *
 * @package gss
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class BaseController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
	
	/**
	 * User Daata
	 *
	 * @var array
	 */
	protected $userData;
	
	/**
	 * Query Settings
	 *
	 * @var array
	 */
	protected $querySettings;
	
	/**
	 * XML Url
	 *
	 * @var string
	 */
	protected $xmlUrl;
	
	/**
	 * XML
	 *
	 * @var string
	 */
	protected $xml;
	
	/**
	 * Doc
	 *
	 * @var DOMDocument $doc
	 */
	protected $doc = false;
	
	/**
	 * Query Terms
	 *
	 * @var array $queryTerms
	 */
	protected $queryTerms = array();
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		
		$this->userData = $this->getUserData();
	}
	
	/**
	 * Get Result
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @param array $arguments
	 * @return array
	 */
	public function getResult($search, $arguments) {
		
		$result = array();
		$this->buildQuerySettings($search, $arguments)->buildXmlUrl()->getXml()->initializeDoc();

		// if there xml has results
		if($this->doc->RES->R) {
			
			$i = 0;
			// handle result items
			foreach($this->doc->RES->R as $item) {
				
				$result['items'][$i] = array(
					'title'		=> $this->replace('title', $this->replaceWrongHtmlOutput((string)$item->T)),
					'url'		=> (string)$item->U,
					'displayUrl'	=> $this->makeDisplayUrl((string)$item->U),
					'description'	=> $this->replace('description', $this->replaceWrongHtmlOutput((string)$item->S)),
					'promotion'		=> false,
				);
				
				// supports promotions
				if($item->SL_RESULTS) {
					
					$result['items'][$i]['description'] = (string)$item->SL_RESULTS->SL_MAIN->BODY_LINE[0]->BLOCK->T;
					$result['items'][$i]['promotion'] = true;
				}
				
				// if this result is a file, we have to get the file details
				if(isset($item->attributes()->MIME) && strpos($item->attributes()->MIME, 'application') !== false) {
					$result['items'][$i]['file'] = $this->getResultFileDetails($item, $arguments);
				}
				
				$i++;
			}
		}
		
		// insert suggestion in result
		$result['suggestion'] = $this->insertSuggestionInResult($search, $arguments);
		
		// calculate results
		$result['results'] = 0;
		if($result['items'] && isset($this->doc->RES->M)) {

			$result['results'] = (integer)$this->doc->RES->M;
			if($result['results'] > 1000) $result['results'] = 1000;
			
			// store user search history
			if($result['results'] >= 1 && $arguments['page'] == 1) {
				
				$lastSearchQuery = false;
				
				// if there is already a user search history
				if($this->userData['history']) {
					
					$lastSearchQuery = end($this->userData['history']);
					$lastSearchQuery = $lastSearchQuery['query'];
					
				// otherwise we have to create the array
				} else {
					
					$this->userData['history'] = array();
				}
				
				// $lastSearchQuery = false OR $lastSearchQuery is not equal to the current $query
				if(!$lastSearchQuery || strtolower($lastSearchQuery) != strtolower($arguments['query'])) {
				
					array_walk($this->userData['history'], array('\TYPO3\Gss\Controller\SearchController', 'removeKeyByQuery'), $arguments['query']);
					
					$this->userData['history'][] = array(
						'query'		=> ucfirst($arguments['query']),
						'link'		=> $arguments['link'],
					);
					
					$this->userData['history'] = array_values($this->userData['history']);

					$this->setUserData($this->userData);
				}
			}
		}
		
		// results start / stop
		$result['start'] = $this->querySettings['start'] + 1;
		$result['stop'] = $this->querySettings['stop'];
		
		return $result;
	}
	
	/**
	 * Make Display Url
	 *
	 * @param string $url
	 * @return string
	 */
	public function makeDisplayUrl($url) {
		
		$url = str_replace(
			array(
				'http://',
				'https://',
			),
			array(
				'',
				'',
			),
			$url
		);
		
		return $url;	
	}
	
	/**
	 * Remove Key By Query
	 *
	 * @param array $value
	 * @param array $key
	 * @param string $query
	 */
	public function removeKeyByQuery($value, $key, $query) {
		if(strtolower($value['query']) == strtolower($query)) unset($this->userData['history'][$key]);
	}
	
	/**
	 * Get Result File Details
	 *
	 * @param SimpleXML $item
	 * @param array $arguments
	 * @return boolean or array
	 */
	public function getResultFileDetails($item, $arguments) {
		
		$file = false;
		$local = false;
		$url = $item->U;
		$parsedUrl = parse_url($url);
		$filePath = $this->getFilePath($parsedUrl);

		// if the host of the result is defined as localDomain
		if(in_array($parsedUrl['host'], $arguments['localDomains']) && file_exists($filePath)) {
			
			$local = true;
			
			// file extension
			$extension = pathinfo($filePath, PATHINFO_EXTENSION);
			
			// file array
			$file = array(
				'size' => $this->fileSizeFormat($filePath),
				'extension' => $extension,
				'extensionUppercase' => strtoupper($extension),
				'icon' => false
			);
		}
		
		// get remote file details
		if(!$local) {
			
			$extension = str_ireplace('application/', '', (string)$item->attributes()->MIME);
			$file = $this->getRemoteFileDetails($url, $extension);
		}
		
		// include file icon
		if($file) {

			// use custom file icons
			if($this->settings['useCustomIcons']) {
	
				$iconPath = $this->settings['customIconsPath'].$file['extension'].'.'.$this->settings['customIconsType'];
				// include icon if file exists
				if(is_file($iconPath)) {
					
					// file icon
					$file['icon'] = array(
						'src' => $iconPath,
						'alt' => $file['extensionUppercase']. ' icon',
						'width' => $this->settings['customIconsWidth'],
						'height' => $this->settings['customIconsHeight']
					);
				}
					
			// use default file icons
			} else {
				
				$iconPath = 'typo3/gfx/fileicons/'.$file['extension'].'.gif';

				// include icon if file exists
				if(is_file($iconPath)) {
					
					// file icon
					$file['icon'] = array(
						'src' => $iconPath,
						'alt' => $file['extensionUppercase']. ' icon',
						'width' => 18,
						'height' => 16
					);
				}
			}
		}

		return $file;
	}
	
	/**
	 * Get Remote File Details
	 *
	 * @param string $fileUrl
	 * @param string $extension
	 * @return boolean or array
	 */
	 public function getRemoteFileDetails($fileUrl, $extension) {

		$file = false;
		
		// curl unit
		$ch = curl_init($fileUrl);
		curl_setopt($ch, CURLOPT_NOBODY, true);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HEADER, true);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); //not necessary unless the file redirects (like the PHP example we're using here)
		$data = curl_exec($ch);
		curl_close($ch);
		
		// if curl is executing normally
		if($data !== false) {
			
			// file array
			$file = array(
				'size' => false,
				'extension' => $extension,
				'extensionUppercase' => strtoupper($extension),
				'icon' => false
			); 
			
			// if content-length is available
			if(preg_match('/Content-Length: (\d+)/', $data, $matches)) {
				
				$remoteFileSize = (int)$matches[1];
				
				// if the remote file size is not empty
				if(!empty($remoteFileSize)) {
					$file['size'] = $this->fileSizeFormat(NULL, $remoteFileSize);
				}
			}
		}
		
		return $file;
	 }
	
	/**
	 * Get File Path
	 *
	 * @param array $parsedUrl
	 * @return string
	 */
	public function getFilePath($parsedUrl) {
		
		$path = '';
		// path
		if($parsedUrl['path']):
			$path .= $parsedUrl['path'];
		endif;
		
		// query
		if($parsedUrl['query']):
			$path .= '?'.$parsedUrl['query'];
		endif;
		$path = ltrim($path, '/');
		
		return $path;
	}
	
	/**
	 * File Size
	 *
	 * @param string $filePath
	 * @param integer $size
	 * @return array $fileSize
	 */
	public function fileSizeFormat($filePath = NULL, $size = NULL) {
		
		$fileSize = array();
		
		// read filesize
		if(is_null($size)) {
			
			$size = filesize($filePath);
		}
		
		// filesize in bytes
		$fileSize['bytes'] = $size;
		
		// calculate unit and unitsize
		$units = array('B', 'KB', 'MB', 'GB', 'TB');
   		for ($i = 0; $size >= 1024 && $i < 4; $i++) $size /= 1024;
		
		// filesize unit
		$fileSize['unit'] = $units[$i];
		
		// filesize unitsize
		$fileSize['unitSize'] = round($size, 2);
		
		// return array with sizes
		return $fileSize;
	}
	
	/**
	 * Insert Promotions In Result
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @param array $arguments
	 * @param array $result
	 * @return array
	 */
	public function insertPromotionsInResult($search, $arguments, $result) {
		
		if(
			// checks if there are custom promotions configured
			count($search->getPromotions()) &&
			// checks if this is page 1 of the results OR if the setting 'showCustomPromotionsOnEachPage' is set
			($arguments['page'] == 1 || $this->settings['showCustomPromotionsOnEachPage'])
		) {
			
			$promotionsArr = array();
			
			// foreach promotions
			foreach($search->getPromotions() as $promotion) {
				
				$terms = strtolower($promotion->getTerms());
				$terms = explode(',', str_ireplace(array(' ,', ', ', '  '), array(',', ',', ' '), $terms));
				
				// checks if the query matches one of the terms
				if(in_array(strtolower($arguments['query']), $terms)) {
					
					$url = $promotion->getLink();
					
					// intern link
					if(ctype_digit($url))
						$url = $this->getFrontendUri($url);
					
					$newResult = array(
						'title'			=> $promotion->getTitle(),
						'url'			=> $url,
						'displayUrl'	=> str_ireplace(
							array('http://', 'https://'),
							array('', ''),
							$url
						),
						'description' 	=> $promotion->getDescription(),
						'promotion' 	=> 1,
					);
					
					// modifies the description
					if($this->settings['removeLineBreaksInResults']) {
						
						$newResult['description'] = str_replace(
							array('</p><p>', '<br />', '<br>'),
							array(' ', ' ', ' '),
							$newResult['description']
						);
					}
					
					$newResult['description'] = str_replace(
						array('<p>', '</p>'),
						array('', ' '),
						$newResult['description']
					);
					
					// inserts the promotion into the $result array
					if($result['items']):
						array_unshift($result['items'], $newResult);
					else:
						$result['items'][0] = $newResult;
					endif;
					
					// we should increase the result count only if this is lower then 10 (this will avoid some problems in the calculation of the amount of pages)
					if(!$result['results'] || $result['results'] <= 9)
						$result['results']++;
				}
			}
		}
		
		return $result;
	}

	/**
	 * Insert Suggestion In Result
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @param array $arguments
	 * @return array
	 */
	public function insertSuggestionInResult($search, $arguments) {
		
		$return = false;
		
		// if $search has configured custom suggestions
		if($search->getSuggestions()) {
			
			$suggestionArr = array();
			
			// handle custom suggestions
			foreach($search->getSuggestions() as $suggestion) {
				
				$terms = strtolower($suggestion->getTerms());
				$terms = str_ireplace(array(' ,', ', ', '  '), array(',', ',', ' '), $terms);
				$suggest = $suggestion->getSuggestion();
				
				if(!empty($terms) && !empty($suggest))
					$suggestionArr[] = array(
						'terms' => explode(',', $terms),
						'suggestion' => $suggest,
					);
			}
			
			// if there are custom suggestions set
			if($suggestionArr) {
				
				// find the query string in the array
				foreach($suggestionArr as $customSuggestion) {
					
					if(in_array(strtolower($arguments['query']), $customSuggestion['terms']))
						$return = $customSuggestion['suggestion'];
				}
			}
		}
		
		// if there isnt a active custom suggestion, we have to check the xml for Googles suggestion
		if(!$return && isset($this->doc->Spelling->Suggestion)) {
			
			$return = (string)strip_tags($this->doc->Spelling->Suggestion);
		}
		
		// modify the return
		if($return) {
			
			$return = array(
				'term' => $return,
				'link' => $arguments['link'],
			);
			$return['link']['query'] = rawurlencode($return['term']);
		}
		
		return $return;
	}

	/**
	 * Get Refinements
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @return array
	 */
	public function getRefinements($search) {
		
		$refinements = array();
			
		// check if we have already a loaded document
		if($this->doc) {
			
			// check if the document has Facets
			if($this->doc->Context->Facet) {
				
				// use the name of this search engine als default label
				$refinements[] = array(
					'uid'	=> 'default',
					'name'	=> $search->getName(),
				);
			
				// loop Facets
				foreach($this->doc->Context->Facet as $refinement) {
					
					$refinements[] = array(
						'uid'	=> (string)$refinement->FacetItem->label,
						'name'	=> (string)$refinement->FacetItem->anchor_text,
					);
				}
				
				// store the refinements in an session, google doesnt return the facets if there are no results (this will fix that)
				$this->userData['refinements'] = $refinements;
				$this->setUserData($this->userData);
			}
		}
		
		// returns the refinements from the session
		if(!$refinements && $this->userData['refinements'])
			$refinements = $this->userData['refinements'];
	
		return $refinements;
	}
	
	/**
	 * Get Custom Refinements
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @return array
	 */
	public function getCustomRefinements($search) {
		
		$refinements = array();
		
		// check if there are custom refinements set
		if($search->getRefinements()) {
			
			// use the name of this search engine als default label
			$refinements[] = array(
				'uid'	=> 'default',
				'name'	=> $search->getName(),
			);
		
			// handle custom refinements
			foreach($search->getRefinements() as $refinement) {
				
				$refinements[] = array(
					'uid'	=> $refinement->getUid(),
					'name'	=> $refinement->getName(),
				);
			}			
		}
		
		return $refinements;
	}
	
	/**
	 * Search Query Redirects
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @param array $arguments
	 * @return void
	 */
	public function searchQueryRedirects($search, $arguments) {
	
		$redirectArr = array();
		
		// handle redirects
		foreach($search->getRedirects() as $redirect) {
			
			$terms = strtolower($redirect->getTerms());
			$terms = str_ireplace(array(' ,', ', ', '  '), array(',', ',', ' '), $terms);
			$url = $redirect->getRedirect();
			
			if(!empty($terms) && !empty($url))
				$redirectArr[] = array(
					'terms' => explode(',', $terms),
					'url' => $url,
				);
		}
		
		// if there are redirects configured
		if($redirectArr) {
			
			// find the query string in the array
			foreach($redirectArr as $customRedirect) {
				
				if(in_array(strtolower($arguments['query']), $customRedirect['terms'])) {

					if(ctype_digit($customRedirect['url'])) {
						
						$this->redirect(NULL, NULL, NULL, NULL, $customRedirect['url']);
						exit;
						
					} else {
						
						$this->redirectToURI($customRedirect['url']);
						
					}
					exit;
				}
			}
		}
	}
	
	/**
	 * Insert Synonyms
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @param array $arguments
	 * @return array
	 */
	public function insertSynonyms($search, $arguments) {
		
		// this will make an array with all terms of the current query
		$this->queryTerms = array_map('trim', explode(' ', strtolower($arguments['query'])));
		
		// this wil make the array with synonyms
		foreach($search->getSynonyms() as $synonym) {
			
			// checks if the term of this synonym matches one of the query terms
			if($termIndex = in_array(strtolower($synonym->getTerm()), $this->queryTerms)) {
				
				// this will add the given synonyms for this term to the query
				foreach($synonym->getSynonyms() as $keyword) {
				
					$newQueryTerm = $keyword->getTerm();
					
					// exact word
					if($keyword->getIsExactWord()) {
						
						$newQueryTerm = '"'.$newQueryTerm.'"';
					
					// 	exclude
					} else if($keyword->isExcludeFromResults()) {
						
						$newQueryTerm = '-'.$newQueryTerm;
					}
					
					// replaces the original word from the query with the new word
					if($keyword->isReplaceWord()) {
						
						$this->queryTerms[$termIndex - 1] = $newQueryTerm;
						
					// adds the new word
					} else {
						
						$this->queryTerms[] = $newQueryTerm;
					}
				}
			}
		}
		
		$arguments['hiddenQuery'] = implode(' ', $this->queryTerms);
		
		return $arguments;
	}
	
	/**
	 * Build Query Settings
	 *
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @param array $arguments
	 * @return $this
	 */
	public function buildQuerySettings($search, $arguments) {
		
		$this->querySettings = array();
		$this->querySettings['cx'] = $search->getCx();
		$this->querySettings['query'] = $arguments['query'];
		// hidden query
		if($arguments['hiddenQuery']):
			$this->querySettings['query'] = $arguments['hiddenQuery'];
		endif;
		$this->querySettings['resultsPerPage'] = $this->settings['resultsPerPage'];
		$this->querySettings['xmlUrl'] = $this->settings['xmlUrl'];
		$this->querySettings['start'] = 0;
		// calculate start based on current page and resultsPerPage
		if(isset($arguments['page']) && $arguments['page'] > 1):
			$this->querySettings['start'] = $arguments['page'] * $this->settings['resultsPerPage'] - $this->settings['resultsPerPage'];
		endif;
		$this->querySettings['stop'] = $this->querySettings['start'] + $this->settings['resultsPerPage'];
		$this->querySettings['refinement'] = false;
		// if some refinement is active
		if($arguments['refinement'] != 'default'):
			$this->querySettings['refinement'] = $arguments['refinement'];
		endif;
		// check if a custom refinement is set
		if($arguments['customRefinement'] != 'default'):
			$customRefinement = $this->refinementRepository->findOneByUid($arguments['customRefinement']);
			if($customRefinement):
				// insert custom refinement in query
				$this->querySettings['query'] = $this->insertCustomRefinementInQuery($this->querySettings['query'], $customRefinement);
			endif;
		endif;
		
		return $this;
	}
	
	/**
	 * Insert Custom Refinement In Query
	 *
	 * @param string $query
	 * @param \TYPO3\Gss\Domain\Model\Refinement $refinement
	 * @return string
	 */
	public function insertCustomRefinementInQuery($query, $refinement) {
		
		$newQuery = '';
		
		// insert refinement urls
		if($refinement->getUrls()) {
			
			$i = 0;
			// handle urls
			foreach($refinement->getUrls() as $url) {
				
				$url = $url->getUrl();
				// insert site in the query
				if(!empty($url)) {
					
					// insert OR in the query
					if($i > 0) $newQuery .= 'OR ';
					
					$newQuery .= 'site:'.$url.' ';				
					$i++;
				}
			}
		}
		
		// insert refinement keywords
		if($refinement->getKeywords()) {
			
			// handle keywords
			foreach($refinement->getKeywords() as $keyword) {
				
				$term = $keyword->getTerm();
				
				// if the term is not empty
				if(!empty($term)) {
					
					// exact term
					if($keyword->isIsExactWord())
						$newQuery .= '"'.$term.'" ';
						
					// exclude term
					else if($keyword->isExcludeFromResults())
						$newQuery .= '-'.$term.' ';
						
					// include term
					else
						$newQuery .= $term.' ';
				}
			}
		}
		
		// insert users query into the the new query
		$newQuery .= $query;
		
		return $newQuery;
	}
	
	/**
	 * Build Xml Url
	 *
	 * @return $this
	 */
	public function buildXmlUrl() {
		
		$this->xmlUrl = str_ireplace(
			array(
				'{cx}',
				'{start}',
				'{resultsPerPage}',
				'{query}',
			),
			array(
				$this->querySettings['cx'],
				$this->querySettings['start'],
				$this->querySettings['resultsPerPage'],
				rawurlencode($this->querySettings['query']),
			),
			$this->querySettings['xmlUrl']
		);
		
		// define refinement
		if($this->querySettings['refinement']) {
			$this->xmlUrl .= '&hq=more:'.$this->querySettings['refinement'];			
		}
		
		return $this;
	}
	
	/**
	 * Get Xml
	 *
	 * @return $this
	 */
	public function getXml() {
		
		$this->xml = GeneralUtility::getURL($this->xmlUrl);
		return $this;
	}
	
	/**
	 * Initialize Doc
	 *
	 * @return $this
	 */
	public function initializeDoc() {
		
		$this->doc = simplexml_load_string($this->xml);
		return $this;
	}
	
	/**
	 * Replace Wrong Html Output
	 *
	 * @param string $content
	 * @return string
	 */
	public function replaceWrongHtmlOutput($content) {
		
		$content = str_ireplace(
			array('<b>',		'</b>',			'<br>'),
			array('<strong>',	'</strong>',	'<br />'),
			$content
		);
		
		// remove linebreaks
		if($this->settings['removeLineBreaksInResults']) {
		
			$content = str_ireplace('<br />', '', $content);	
		}
		
		return $content;
	}
	
	/**
	 * Replace
	 *
	 * @param string $field
	 * @param string $value
	 *
	 * @return void
	 */
	public function replace($field, $value) {

		if(
			$this->settings['replaces'][$field]['search'] &&
			$this->settings['replaces'][$field]['replace'] &&
			is_array($this->settings['replaces'][$field]['search']) &&
			count($this->settings['replaces'][$field]['search']) == count($this->settings['replaces'][$field]['replace'])
		) {
			
			$value = str_ireplace($this->settings['replaces'][$field]['search'], $this->settings['replaces'][$field]['replace'], $value);
		}
			
		return $value;
	}
	
	/**
	 * Insert Link And Form Arguments
	 *
	 * @param array $arguments
	 * @return array
	 */
	public function insertLinkAndFormArguments($arguments) {
		
		$arguments['form'] = array();
		$arguments['form']['refinement'] = $arguments['refinement'];
		$arguments['form']['customRefinement'] = $arguments['customRefinement'];
		
		$arguments['link'] = array();
		$arguments['link']['query'] = $arguments['queryEncoded'];
		
		// replaces the space character with the character from the setting (this will fix some issues in the Safari browser)
		if($this->settings['querySpaceCharacter']) {
			$arguments['link']['query'] = rawurlencode(str_replace(' ', $this->settings['querySpaceCharacter'], rawurldecode($arguments['queryEncoded'])));
		}
		
		// insert refinement argument when its not default
		if($arguments['refinement'] != 'default') {
			$arguments['link']['refinement'] = $arguments['refinement'];
		}
		// insert cutom refinement argument when its not default
		if($arguments['customRefinement'] != 'default') {
			$arguments['link']['customRefinement'] = $arguments['customRefinement'];
		}
		
		return $arguments;
	}
	
	/**
	 * Insert Local Domains
	 *
	 * @param array $arguments
	 * @param \TYPO3\Gss\Domain\Model\Search $search
	 * @return array
	 */
	public function insertLocalDomains($arguments, $search) {
		
		$arguments['localDomains'] = false;
		
		// if there are local domains configured
		if($search->getLocalDomains()) {
			
			$arguments['localDomains'] = array();
			
			// handle local domains
			foreach($search->getLocalDomains() as $localDomain) {
				
				$arguments['localDomains'][] = $localDomain->getName();
			}
		}
		
		return $arguments;
	}
	
	/**
	 * Page Browser
	 *
	 * @param integer $results
	 * @param integer $resultsPerPage
	 * @param integer $currentPage
	 * @param array $linkArguments
	 * @param integer $range
	 * @param integer $buttons 
	 * @return array $result
	 */
	public function pageBrowser($currentPage, $results, $resultsPerPage, $linkArguments = NULL, $buttons = true, $range = 5) {

		if(is_null($linkArguments)) {
			$linkArguments = array();	
		}
		if($results <= $resultsPerPage):
			$result['pages'] = 1;
		else:
		
			$result['pages'] = ceil($results / $resultsPerPage);
			$currentPage = (empty($currentPage) || $currentPage > $result['pages']) ? 1 : $currentPage;
			$result['current'] = $currentPage;

			if($currentPage > 1):
				$result['prev'] = $currentPage - 1;
				$linkArgument = $linkArguments;
				$linkArgument['page'] = $result['prev'];
				if($buttons)
					$result['menu'][] = array('label' => '<', 'page' => $result['prev'], 'class' => 'prev', 'linkArgument' => $linkArgument);
			endif; 

			$i = 1;
			while($i <= $result['pages']):
			
				if($i == 1 || $i == $result['pages'] || $result['pages'] < 12 || ($i >= $currentPage - $range && $i <= $currentPage + $range)):
				
					$class = '';
					if($i == 1) $class .= 'first ';
					if($i == $result['pages']) $class .= 'last ';
					if($i == $currentPage) $class .= 'selected ';
					$linkArgument = $linkArguments;
					$linkArgument['page'] = $i;
					$result['menu'][] = array('label' => $i, 'page' => $i, 'class' => $class, 'linkArgument' => $linkArgument);
				endif;
				$i++;
			endwhile;
			
			if($currentPage < $result['pages']):
				$result['next'] = $currentPage + 1;
				$linkArgument = $linkArguments;
				$linkArgument['page'] = $result['next'];
				if($buttons)
					$result['menu'][] = array('label' => '>', 'page' => $result['next'], 'class' => 'next', 'linkArgument' => $linkArgument);
			endif; 
		endif;
		
		$result['start'] = $currentPage * $resultsPerPage - $resultsPerPage;
		$result['stop'] = $result['start'] + $resultsPerPage;
 		if($result['pages'] == 1 || $result['start'] <= 0)
			$result['start'] = 0;
		$result['results'] = $results;
		$result['curPage'] = $GLOBALS['TSFE']->id;
		
		return $result;
	}
	
	/**
	 * Get user data
	 *
	 * @return void
	 */
	public function getUserData() {
		
		return $GLOBALS["TSFE"]->fe_user->getKey('ses', 'tx_gss');
	}
	
	/**
	 * Set user data
	 *
	 * @param array $data
	 * @return void
	 */
	public function setUserData($data) {
		
		// Sets the user session from
		$this->userData = $data;
		$GLOBALS["TSFE"]->fe_user->setKey('ses', 'tx_gss', $data);
		$GLOBALS['TSFE']->fe_user->storeSessionData();
	}
	
	/**
	 * Get Search History
	 *
	 * @return array
	 */
	public function getUserSearchHistory() {
		
		$searchHistory = array();
		
		// if the user search history exist
		if($this->userData['history'] && $this->settings['searchHistory']['maxItems']) {
			
			$i = 0;
			if(count($this->userData['history']) < $this->settings['searchHistory']['maxItems']) {
			
				$this->settings['searchHistory']['maxItems'] = 	count($this->userData['history']);
			}
			
			while($i < $this->settings['searchHistory']['maxItems']) {
				
				// item to collect
				$itemNo = count($this->userData['history']) - $i - 1;
				$searchHistory[] = $this->userData['history'][$itemNo];
				
				$i++;	
			}
		}
		
		return $searchHistory;
	}
	
	/**
	 * Get Autocompletion Suggestions
	 *
	 * @param string $cx
	 * @param string $term
	 * @return $this
	 */
	public function getAutocompletionSuggestions($cx, $term) {
		
		$jsonUrl = str_ireplace(
			array(
				'{query}',
				'{cx}',
			),
			array(
				rawurlencode($term),
				$cx,
			),
			$this->settings['jsonUrl']
		);
		
		// get $jsonUrl
		$response = GeneralUtility::getURL($jsonUrl);
		
		// remove javascript call back function
		$response = str_ireplace('window.google.ac.h(', '', $response);
		$response = rtrim($response, ')');
	
		// response array
		$response = json_decode($response);
		
		// return response
		if($response[1]) {
			
			$suggestions = array();
			
			// foreach suggestions
			foreach($response[1] as $suggest) {
				
				$suggestions[] = $suggest[0];	
			}
			
			return $suggestions;
			
		} else {
			
			return array();
		}
	}
	
	/**
	 * Get Frontend Uri
	 *
	 * @param integer $pageUid
	 * @param array $additionalParams
	 * @return string
	 */
	protected function getFrontendUri($pageUid, array $additionalParams = array()) {
		
		// website baseurl
		$baseUrl = rtrim($GLOBALS['TSFE']->baseUrl, '/').'/';
		
		// get uri builder
		$uriBuilder = $this->controllerContext->getUriBuilder();

		$uri = $uriBuilder
		// set target page uid
		->setTargetPageUid($pageUid)
		// set use cache hash
		->setUseCacheHash(true)
		// set arguments
		->setArguments($additionalParams)
		// build
		->build();
			
		return rawurldecode($baseUrl.ltrim($uri, '/'));
	}
}