<?php
namespace TYPO3\Gss\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Arek van Schaijk <info@ucreation.nl>, Ucreation
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 *
 *
 * @package gss
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class SearchController extends BaseController {
	
	/**
	 * searchRepository
	 * 
	 * @var \TYPO3\Gss\Domain\Repository\SearchRepository
	 * @inject
	 */
	protected $searchRepository = NULL;
	
	/**
	 * refinementRepository
	 * 
	 * @var \TYPO3\Gss\Domain\Repository\RefinementRepository
	 * @inject
	 */
	protected $refinementRepository = NULL;

	/**
	 * action show
	 *
	 * @return void
	 */
	public function renderAction() {
		
		// if there is a search configured
		if(!empty($this->settings['searchUid'])) {
			
			$search = $this->searchRepository->findOneByUid($this->settings['searchUid']);
			
			// if the search exists
			if($search) {
			
				// get arguments
				$arguments = $this->request->getArguments();
				if($arguments['query']):
				
					// removes the special space character back to a normal space character if defined as settings
					if($this->settings['querySpaceCharacter'])
						$arguments['query'] = str_replace($this->settings['querySpaceCharacter'], ' ', $arguments['query']);
						
					$arguments['queryEncoded'] = rawurlencode(rawurldecode($arguments['query']));					
					$arguments['query'] = rawurldecode($arguments['query']);
				endif;
				if(!isset($arguments['refinement']) || empty($arguments['refinement'])):
					$arguments['refinement'] = 'default';
				endif;
				if(!isset($arguments['customRefinement']) || empty($arguments['customRefinement'])):
					$arguments['customRefinement'] = 'default';
				endif;
				if(!isset($arguments['page']) || empty($arguments['page']) || !ctype_digit($arguments['page'])) {
					$arguments['page'] = 1;	
				}
				
				// insert link and form arguments
				$arguments = $this->insertLinkAndFormArguments($arguments);
				
				// insert local domains
				$arguments = $this->insertLocalDomains($arguments, $search);

				// redirects
				if($arguments['query'] && $this->settings['redirects'] && $search->getRedirects())
					$this->searchQueryRedirects($search, $arguments);
				
				// synonyms
				if($arguments['query'] && count($search->getSynonyms()))
					$arguments = $this->insertSynonyms($search, $arguments);

				// get results
				if($arguments['query'] && $this->settings['result']):
					$result = $this->getResult($search, $arguments);
					// includes the custom promotions
					$result = $this->insertPromotionsInResult($search, $arguments, $result);
					// pagebrowser
					if(!$this->settings['disablePagebrowser']) {
						$result['pageBrowser'] = $this->pageBrowser(
							$arguments['page'],
							$result['results'],
							$this->settings['resultsPerPage'],
							$arguments['link'],
							$this->settings['pagebrowser']['enableButtons'],
							$this->settings['pagebrowser']['range']
						);
					}
					$this->view->assign('result', $result);
				endif;
				
				// get refinements
				if($arguments['query'] && $this->settings['refinements']):
					// use refinements from Google
					if($search->isUseGooglesRefinements()):
						$refinements = $this->getRefinements($search);
					elseif(count($search->getRefinements())):
						$refinements = $this->getCustomRefinements($search);
					endif;
					// TODO: Fix this temporary workaround
					if(count($refinements) > 1) {
						$this->view->assign('refinements', $refinements);
					}
				endif;
				
				// get search history
				if($this->settings['searchHistory']) {
					$searchHistory = $this->getUserSearchHistory();
					$this->view->assign('searchHistory', $searchHistory);	
				}
				
				$this->view->assign('search', $search);
				$this->view->assign('arguments', $arguments);
			
			// the search doesnt exists
			} else {
				
				$this->view->assign('error', 'search_doesnt_exists');
			}
			
		// there is no search configured
		} else {
			
			$this->view->assign('error', 'no_search_configured');	
		}
	}
	
	/**
	 * action post
	 *
	 * @return void
	 */
	public function postAction() {
		
		// get arguments
		$arguments = $this->request->getArguments();
		
		// modify query string
		if($arguments['query']) {
			
			$arguments['query'] = trim($arguments['query']);
			
			// replaces the space character with the character from the setting (this will fix some issues in the Safari browser)
			if($this->settings['querySpaceCharacter']) {
                $arguments['query'] = str_replace(' ', $this->settings['querySpaceCharacter'], $arguments['query']);
            }
			
		} else {
			
			$arguments['query'] = '';
		}
		
		// when the query string has some value
		if(!empty($arguments['query'])) {
			
			// if a refinement is set
			if(isset($arguments['refinement']) && !empty($arguments['refinement']) && $arguments['refinement'] != 'default') {
				
				$this->redirect(NULL, NULL, NULL, array('query' => $arguments['query'], 'refinement' => $arguments['refinement']));
				
			// if a custom refinement is set
			} else if(isset($arguments['customRefinement']) && !empty($arguments['customRefinement']) && $arguments['customRefinement'] != 'default') {
				
				$this->redirect(NULL, NULL, NULL, array('query' => $arguments['query'], 'customRefinement' => $arguments['customRefinement']));
				
			// when there are no refinements set
			} else {
				
				$this->redirect(NULL, NULL, NULL, array('query' => $arguments['query']));
			}
		} else {
			
			$this->redirect(NULL, NULL, NULL, NULL);
		}
		exit;
	}
	
	/**
	 * Autocompletion Action
	 *
	 * return string
	 */
	 public function autocompletionAction() {
		
		$suggestions = array();
		
		// expected parameters
		if(GeneralUtility::_GP('term') && GeneralUtility::_GP('searchid')) {
			
			$term = GeneralUtility::_GP('term');
			$searchUid = GeneralUtility::_GP('searchid');
			
			// validate search id
			if(!empty($searchUid) && ctype_digit($searchUid)) {
				
				// find one by uid
				$search = $this->searchRepository->findOneByUid($searchUid);
				
				// if the search exists
				if($search)
					$suggestions = $this->getAutocompletionSuggestions($search->getCx(), $term);	
			}
		}
		
		// strip underscore results if set
		if($this->settings['stripUnderscoreResultsFromAutocompletion']) {
			
			foreach($suggestions as $key => $value) {
				
				if(strpos($value, '__') !== false)
					unset($suggestions[$key]);
			}
		}
		
		echo json_encode($suggestions);
		exit;
	 }
}