<?php
namespace TYPO3\Gss\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Arek van Schaijk <info@ucreation.nl>, Ucreation
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package gss
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Refinement extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

	/**
	 * name
	 *
	 * @var string
	 */
	protected $name;

	/**
	 * urls
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Url>
	 */
	protected $urls;

	/**
	 * keywords
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Keyword>
	 */
	protected $keywords;

	/**
	 * __construct
	 *
	 * @return void
	 */
	public function __construct() {
		//Do not remove the next line: It would break the functionality
		$this->initStorageObjects();
	}

	/**
	 * Initializes all \TYPO3\CMS\Extbase\Persistence\ObjectStorage properties.
	 *
	 * @return void
	 */
	protected function initStorageObjects() {
		/**
		 * Do not modify this method!
		 * It will be rewritten on each save in the extension builder
		 * You may modify the constructor of this class instead
		 */
		$this->urls = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->keywords = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
	}

	/**
	 * Returns the name
	 *
	 * @return string $name
	 */
	public function getName() {
		return $this->name;
	}

	/**
	 * Sets the name
	 *
	 * @param string $name
	 * @return void
	 */
	public function setName($name) {
		$this->name = $name;
	}

	/**
	 * Adds a Url
	 *
	 * @param \TYPO3\Gss\Domain\Model\Url $url
	 * @return void
	 */
	public function addUrl(\TYPO3\Gss\Domain\Model\Url $url) {
		$this->urls->attach($url);
	}

	/**
	 * Removes a Url
	 *
	 * @param \TYPO3\Gss\Domain\Model\Url $urlToRemove The Url to be removed
	 * @return void
	 */
	public function removeUrl(\TYPO3\Gss\Domain\Model\Url $urlToRemove) {
		$this->urls->detach($urlToRemove);
	}

	/**
	 * Returns the urls
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Url> $urls
	 */
	public function getUrls() {
		return $this->urls;
	}

	/**
	 * Sets the urls
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Url> $urls
	 * @return void
	 */
	public function setUrls(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $urls) {
		$this->urls = $urls;
	}

	/**
	 * Adds a Keyword
	 *
	 * @param \TYPO3\Gss\Domain\Model\Keyword $keyword
	 * @return void
	 */
	public function addKeyword(\TYPO3\Gss\Domain\Model\Keyword $keyword) {
		$this->keywords->attach($keyword);
	}

	/**
	 * Removes a Keyword
	 *
	 * @param \TYPO3\Gss\Domain\Model\Keyword $keywordToRemove The Keyword to be removed
	 * @return void
	 */
	public function removeKeyword(\TYPO3\Gss\Domain\Model\Keyword $keywordToRemove) {
		$this->keywords->detach($keywordToRemove);
	}

	/**
	 * Returns the keywords
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Keyword> $keywords
	 */
	public function getKeywords() {
		return $this->keywords;
	}

	/**
	 * Sets the keywords
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Keyword> $keywords
	 * @return void
	 */
	public function setKeywords(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $keywords) {
		$this->keywords = $keywords;
	}

}