<?php
namespace TYPO3\Gss\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Arek van Schaijk <info@ucreation.nl>, Ucreation
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package gss
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Search extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

	/**
	 * name
	 *
	 * @var string
	 * @validate NotEmpty
	 */
	protected $name;

	/**
	 * description
	 *
	 * @var string
	 */
	protected $description;

	/**
	 * cx
	 *
	 * @var string
	 * @validate NotEmpty
	 */
	protected $cx;

	/**
	 * enableAutocompletion
	 *
	 * @var boolean
	 */
	protected $enableAutocompletion = false;

	/**
	 * suggestions
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Suggestion>
	 */
	protected $suggestions;

	/**
	 * useGooglesRefinements
	 *
	 * @var boolean
	 */
	protected $useGooglesRefinements = false;

	/**
	 * refinements
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Refinement>
	 */
	protected $refinements;

	/**
	 * promotions
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Promotion>
	 */
	protected $promotions;

	/**
	 * timePeriods
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Tx_Gss_Domain_Model_TimePeriod>
	 */
	protected $timePeriods;

	/**
	 * redirects
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Redirect>
	 */
	protected $redirects;

	/**
	 * synonyms
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Synonym>
	 */
	protected $synonyms;
	
	/**
	 * localDomains
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Domain>
	 */
	protected $localDomains;

	/**
	 * __construct
	 *
	 * @return void
	 */
	public function __construct() {
		//Do not remove the next line: It would break the functionality
		$this->initStorageObjects();
	}

	/**
	 * Initializes all \TYPO3\CMS\Extbase\Persistence\ObjectStorage properties.
	 *
	 * @return void
	 */
	protected function initStorageObjects() {
		/**
		 * Do not modify this method!
		 * It will be rewritten on each save in the extension builder
		 * You may modify the constructor of this class instead
		 */
		$this->suggestions = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->refinements = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->promotions = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->timePeriods = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->redirects = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->synonyms = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		
		$this->localDomains = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
	}

	/**
	 * Returns the name
	 *
	 * @return string $name
	 */
	public function getName() {
		return $this->name;
	}

	/**
	 * Sets the name
	 *
	 * @param string $name
	 * @return void
	 */
	public function setName($name) {
		$this->name = $name;
	}

	/**
	 * Returns the description
	 *
	 * @return string $description
	 */
	public function getDescription() {
		return $this->description;
	}

	/**
	 * Sets the description
	 *
	 * @param string $description
	 * @return void
	 */
	public function setDescription($description) {
		$this->description = $description;
	}

	/**
	 * Returns the cx
	 *
	 * @return string $cx
	 */
	public function getCx() {
		return $this->cx;
	}

	/**
	 * Sets the cx
	 *
	 * @param string $cx
	 * @return void
	 */
	public function setCx($cx) {
		$this->cx = $cx;
	}

	/**
	 * Returns the enableAutocompletion
	 *
	 * @return boolean $enableAutocompletion
	 */
	public function getEnableAutocompletion() {
		return $this->enableAutocompletion;
	}

	/**
	 * Sets the enableAutocompletion
	 *
	 * @param boolean $enableAutocompletion
	 * @return void
	 */
	public function setEnableAutocompletion($enableAutocompletion) {
		$this->enableAutocompletion = $enableAutocompletion;
	}

	/**
	 * Returns the boolean state of enableAutocompletion
	 *
	 * @return boolean
	 */
	public function isEnableAutocompletion() {
		return $this->getEnableAutocompletion();
	}

	/**
	 * Adds a Suggestion
	 *
	 * @param \TYPO3\Gss\Domain\Model\Suggestion $suggestion
	 * @return void
	 */
	public function addSuggestion(\TYPO3\Gss\Domain\Model\Suggestion $suggestion) {
		$this->suggestions->attach($suggestion);
	}

	/**
	 * Removes a Suggestion
	 *
	 * @param \TYPO3\Gss\Domain\Model\Suggestion $suggestionToRemove The Suggestion to be removed
	 * @return void
	 */
	public function removeSuggestion(\TYPO3\Gss\Domain\Model\Suggestion $suggestionToRemove) {
		$this->suggestions->detach($suggestionToRemove);
	}

	/**
	 * Returns the suggestions
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Suggestion> $suggestions
	 */
	public function getSuggestions() {
		return $this->suggestions;
	}

	/**
	 * Sets the suggestions
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Suggestion> $suggestions
	 * @return void
	 */
	public function setSuggestions(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $suggestions) {
		$this->suggestions = $suggestions;
	}
	
	/**
	 * Returns the useGooglesRefinements
	 *
	 * @return boolean $useGooglesRefinements
	 */
	public function getUseGooglesRefinements() {
		return $this->useGooglesRefinements;
	}

	/**
	 * Sets the useGooglesRefinements
	 *
	 * @param boolean $useGooglesRefinements
	 * @return void
	 */
	public function setUseGooglesRefinements($useGooglesRefinements) {
		$this->useGooglesRefinements = $useGooglesRefinements;
	}

	/**
	 * Returns the boolean state of useGooglesRefinements
	 *
	 * @return boolean
	 */
	public function isUseGooglesRefinements() {
		return $this->getUseGooglesRefinements();
	}

	/**
	 * Adds a Refinement
	 *
	 * @param \TYPO3\Gss\Domain\Model\Refinement $refinement
	 * @return void
	 */
	public function addRefinement(\TYPO3\Gss\Domain\Model\Refinement $refinement) {
		$this->refinements->attach($refinement);
	}

	/**
	 * Removes a Refinement
	 *
	 * @param \TYPO3\Gss\Domain\Model\Refinement $refinementToRemove The Refinement to be removed
	 * @return void
	 */
	public function removeRefinement(\TYPO3\Gss\Domain\Model\Refinement $refinementToRemove) {
		$this->refinements->detach($refinementToRemove);
	}

	/**
	 * Returns the refinements
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Refinement> $refinements
	 */
	public function getRefinements() {
		return $this->refinements;
	}

	/**
	 * Sets the refinements
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Refinement> $refinements
	 * @return void
	 */
	public function setRefinements(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $refinements) {
		$this->refinements = $refinements;
	}

	/**
	 * Adds a Promotion
	 *
	 * @param \TYPO3\Gss\Domain\Model\Promotion $promotion
	 * @return void
	 */
	public function addPromotion(\TYPO3\Gss\Domain\Model\Promotion $promotion) {
		$this->promotions->attach($promotion);
	}

	/**
	 * Removes a Promotion
	 *
	 * @param \TYPO3\Gss\Domain\Model\Promotion $promotionToRemove The Promotion to be removed
	 * @return void
	 */
	public function removePromotion(\TYPO3\Gss\Domain\Model\Promotion $promotionToRemove) {
		$this->promotions->detach($promotionToRemove);
	}

	/**
	 * Returns the promotions
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Promotion> $promotions
	 */
	public function getPromotions() {
		return $this->promotions;
	}

	/**
	 * Sets the promotions
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Promotion> $promotions
	 * @return void
	 */
	public function setPromotions(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $promotions) {
		$this->promotions = $promotions;
	}

	/**
	 * Adds a TimePeriod
	 *
	 * @param Tx_Gss_Domain_Model_TimePeriod $timePeriod
	 * @return void
	 */
	public function addTimePeriod(Tx_Gss_Domain_Model_TimePeriod $timePeriod) {
		$this->timePeriods->attach($timePeriod);
	}

	/**
	 * Removes a TimePeriod
	 *
	 * @param Tx_Gss_Domain_Model_TimePeriod $timePeriodToRemove The TimePeriod to be removed
	 * @return void
	 */
	public function removeTimePeriod(Tx_Gss_Domain_Model_TimePeriod $timePeriodToRemove) {
		$this->timePeriods->detach($timePeriodToRemove);
	}

	/**
	 * Returns the timePeriods
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Tx_Gss_Domain_Model_TimePeriod> $timePeriods
	 */
	public function getTimePeriods() {
		return $this->timePeriods;
	}

	/**
	 * Sets the timePeriods
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Tx_Gss_Domain_Model_TimePeriod> $timePeriods
	 * @return void
	 */
	public function setTimePeriods(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $timePeriods) {
		$this->timePeriods = $timePeriods;
	}

	/**
	 * Adds a Redirect
	 *
	 * @param \TYPO3\Gss\Domain\Model\Redirect $redirect
	 * @return void
	 */
	public function addRedirect(\TYPO3\Gss\Domain\Model\Redirect $redirect) {
		$this->redirects->attach($redirect);
	}

	/**
	 * Removes a Redirect
	 *
	 * @param \TYPO3\Gss\Domain\Model\Redirect $redirectToRemove The Redirect to be removed
	 * @return void
	 */
	public function removeRedirect(\TYPO3\Gss\Domain\Model\Redirect $redirectToRemove) {
		$this->redirects->detach($redirectToRemove);
	}

	/**
	 * Returns the redirects
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Redirect> $redirects
	 */
	public function getRedirects() {
		return $this->redirects;
	}

	/**
	 * Sets the redirects
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Redirect> $redirects
	 * @return void
	 */
	public function setRedirects(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $redirects) {
		$this->redirects = $redirects;
	}

	/**
	 * Adds a Synonym
	 *
	 * @param \TYPO3\Gss\Domain\Model\Synonym $synonym
	 * @return void
	 */
	public function addSynonym(\TYPO3\Gss\Domain\Model\Synonym $synonym) {
		$this->synonyms->attach($synonym);
	}

	/**
	 * Removes a Synonym
	 *
	 * @param \TYPO3\Gss\Domain\Model\Synonym $synonymToRemove The Synonym to be removed
	 * @return void
	 */
	public function removeSynonym(\TYPO3\Gss\Domain\Model\Synonym $synonymToRemove) {
		$this->synonyms->detach($synonymToRemove);
	}

	/**
	 * Returns the synonyms
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Synonym> $synonyms
	 */
	public function getSynonyms() {
		return $this->synonyms;
	}

	/**
	 * Sets the synonyms
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Synonym> $synonyms
	 * @return void
	 */
	public function setSynonyms(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $synonyms) {
		$this->synonyms = $synonyms;
	}
	
	/**
	 * Returns the localDomains
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Domain> $localDomains
	 */
	public function getLocalDomains() {
		return $this->localDomains;
	}

	/**
	 * Sets the localDomains
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\Gss\Domain\Model\Domain> $localDomains
	 * @return void
	 */
	public function setLocalDomains(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $localDomains) {
		$this->localDomains = $localDomains;
	}

}